/*
This is the DOS side of Glidos's Redbook support.
All programs that run under Glidos use the DOS/4GW
extender to run in protected mode.  Hence they
perform realmode int 2fh's via the int 31h DPMI
call, and Glidos sits on the 31h vector.
Someone wishing to support arbitrary programs
should sit on int 2fh directly from real mode.

This is not GPL'd.  You can do whatever you like
with it, except hold me liable for any problems
arising from its use.
*/

#define NCDCMD (32)

struct real_call
{
    int edi;
    int esi;
    int ebp;
    int reserved;
    int ebx;
    int edx;
    int ecx;
    int eax;
    short flags;
    short es;
    short ds;
    short fs;
    short gs;
    short ip;
    short cs;
    short sp;
    short ss;
};

struct cd_cmd
{
    int drive;
    int start;
    int count;
};

struct cd_cmd cd_cmd_buf[NCDCMD];
int cd_cmd_n = 0;


void (__interrupt __far *old_int31_handler)();

void __interrupt __far int31_handler(union INTPACK r)
{
    if((r.x.eax & 0xffff) == 0x0300 && r.x.ebx == 0x2f)
    {
        /* We have picked up a simulated real mode int 2fh
         * Lets have a look at the call structure */
        struct real_call *rc = (struct real_call *) r.x.edi;

        if(rc->eax == 0x1510)
        {
            /* Its a device request aimed at mscdex
             * es:bx is the real mode address of the
             * request header */
            unsigned char *req = (unsigned char *) ((rc->es)<<4 | rc->ebx);


            if(req[2] == 131) /* Seek */
            {
                /* Return success indication */
                req[3] = 0;
                req[4] = 1;

                return;
            }

            if(req[2] == 132 && cd_cmd_n < NCDCMD) /* Play Audio */
            {
                cd_cmd_buf[cd_cmd_n].drive = r.x.ecx;
                cd_cmd_buf[cd_cmd_n].start = req[14] | (req[15] << 8) | (req[16] << 16) | (req[17] << 24);
                cd_cmd_buf[cd_cmd_n].count = req[18] | (req[19] << 8) | (req[20] << 16) | (req[21] << 24);
                
                cd_cmd_n += 1;

                /* Return success indication */
                req[3] = 0;
                req[4] = 1;

                return;
            }

            /* Stop Audio */
            /* Let TR handle the stop command, but only
             * because there is a delay in Glidos that
             * leads to TR attempting file io before
             * playback has finished. */
            /*
            if(req[2] == 133 && cd_cmd_n < NCDCMD)
            {
                chain(old_int31_handler);
            }
            */

            if(req[2] == 3 && cd_cmd_n < NCDCMD) /* IOCTL READ */
            {
                cd_cmd_buf[cd_cmd_n].drive = r.x.ecx;
                cd_cmd_buf[cd_cmd_n].start = 0;
                cd_cmd_buf[cd_cmd_n].count = 0;
                
                cd_cmd_n += 1;

                /* Return success indication */
                req[3] = 0;
                req[4] = 1;

                return;
            }
        }
    }

    /* In cases where we don't deal with request,
     * we chain to the original handler */
    chain(old_int31_handler);
}


/* This is where the commands get sent off
 * to the Windows side server
 */
static void send_cd_cmds(void)
{
    int i;
    
    for(i = 0; i < cd_cmd_n; i++)
    {
        if(cd_cmd_buf[i].start == 0)
        {
            send_char(CMD_AudioPoll);
        }
        else
        {
            send_char(CMD_AudioPlay);
            send_int(cd_cmd_buf[i].drive);
            send_int(cd_cmd_buf[i].start);
            send_int(cd_cmd_buf[i].count);
        }
    }
    
    cd_cmd_n = 0;
}


static int sat_on_31 = 0;

static void sit_on_31(void)
{
    if(!sat_on_31)
    {
        sat_on_31 = 1;

        old_int31_handler = getvect(0x31);
        setvect(0x31, int31_handler);
    }
}

static void get_off_31(void)
{
    if(sat_on_31)
    {
        sat_on_31 = 0;
        setvect(0x31, old_int31_handler);
    }
}

